#ifndef __ZEOLITE_DEFINES_INCLUDED__
#define __ZEOLITE_DEFINES_INCLUDED__

#pragma once

// choose your header
#include <stdafx.h> // for MFC (remember to include your project's path in the 'additonal includes' list)
//#include <windows.h>  // for generic Windows
//#include <ctype.h>  // for MinGW

// variable handles

typedef void* ZVAR;
typedef void* ZSTR;
typedef void* ZMAP;
typedef void* ZBUF;
typedef void* ZLIST;
typedef void* ZFUNC;
typedef void* ZFORMAT;

// some selected variable type defines

#define VarID_INVALID		0
#define VarID_void			1
#define VarID_variant		2
#define VarID_bool			3
#define VarID_char			4
#define VarID_byte			5	// unsigned byte
#define VarID_sbyte			104	// signed byte
#define VarID_short			6
#define VarID_ushort		105
#define VarID_int			7
#define VarID_uint			106
#define VarID_int64			8
#define VarID_float			9
#define VarID_double		10
#define VarID_string		11
#define VarID_rect32		12 // a rect32 struct (see below)
#define VarID_colour		13
#define VarID_vector3c		101 // vector of 3 signed characters
#define VarID_vector3i		102 // vector of 3 long integers
#define VarID_vector3f		14  // vector of 3 floats
#define VarID_vector3d		103 // vector of 3 doubles
#define VarID_buffer		15
#define VarID_voidptr		20
#define VarID_varid			23 // long, must be a valid VarID
#define VarID_hvar			25
#define VarID_varref		26
#define VarID_varlist		31
#define VarID_map			60
#define VarID_mapid			61 // long, must be a valid map type ID
#define VarID_format		100
#define VarID_filesel		300
#define VarID_dirsel		301
#define VarID_combosel		302
#define VarID_climate		340
#define VarID_landtype		341
#define VarID_zeofunc		510
#define VarID_progbox		600

// map type defines

#define MAP_Empty				0
#define MAP_BYTE				1
#define MAP_WORD				2
#define MAP_RGB					3
#define MAP_RGBA				4
#define MAP_DWORD				41

#define MAP_DesignMap			10	// DM
#define MAP_Heightfield			20	// HF
#define MAP_TerrainNormals		30	// TN
#define MAP_AttributesMap		40	// AM
#define MAP_WaterMap			50	// WM
#define MAP_AuxWaterMap			52	// AuxWM

// map flags (for use with map_GetFlag/SetFlag)

#define MAPFLAG_READY		1
#define MAPFLAG_SAVED		2
#define MAPFLAG_MOSAIC		3
#define MAPFLAG_INVALID		4
#define MAPFLAG_MODIFIED	5
#define MAPFLAG_SEAMFIX		6

// map allocation modes  (from zmap_GetAllocMode)

#define MAPALLOC_INVALID			0x00000000
#define MAPALLOC_DONTCARE			MAPALLOC_INVALID
#define MAPALLOC_RAM				0x00000001
#define MAPALLOC_MOSAIC				0x00000002 // a true mosaic, with all bells and whistles
#define MAPALLOC_MOSAIC_TEMP		0x00000004 // a temporary mosaic, stored in temp dir. Map is saved as single file.
#define MAPALLOC_MOSAIC_SYSPAGEFILE	0x00000010 // a temporary mosaic, stored in sys page file. Map is saved as single file.
#define MAPALLOC_MOSAIC_MEMMAPFILE	0x00000020 // a temporary mosaic, stored in a mem mapped file. Map is saved as single file.

// format flags (for use with format_SetFlags/format_GetFlag)

#define ZFORMAT_FLAG_NATIVE		0x001 // supported as a nominally lossless load/save format
#define ZFORMAT_FLAG_MOSAIC		0x002
#define ZFORMAT_FLAG_LOAD		0x004
#define ZFORMAT_FLAG_SAVE		0x008
#define ZFORMAT_FLAG_LOADTILE	0x010
#define ZFORMAT_FLAG_SAVETILE	0x020

// menu contexts and flags

#define ZMENU_CONTEXT_EXT_CUR		0
#define ZMENU_CONTEXT_EXT_ROOT		1
#define ZMENU_CONTEXT_FILE_IMPORT	2
#define ZMENU_CONTEXT_FILE_EXPORT	3

// flags for progress box

#define ZPROGBOX_SHOW_PERCENT	0x1
#define ZPROGBOX_INDEFINITE		0x2
#define ZPROGBOX_ALLOW_CANCEL	0x4

// function exports name decoration

#define zeoexport extern "C" __declspec( dllexport )


// useful constants

#ifndef PI
	#define PI 3.14159265
#endif

#ifndef NULL
	#define NULL 0
#endif

// useful structs

typedef struct { signed char x, y, z; } vector3c;
typedef struct { float x, y, z; } vector3f;
typedef struct { long x, y, z; } vector3i;
typedef struct { double x, y, z; } vector3d;

typedef struct { vector3f i, j, k; } matrix3x3f;
typedef struct { vector3d i, j, k; } matrix3x3d;

typedef struct { int x1, x2, y1, y2; } rect32; // x2 and y2 are inclusive by definition (different to RECT)

// pixel type structs

typedef struct {
	short int  Alt;		// exact altitude (in m) (-32k -> 32k)
	unsigned char Peak;			// cone strength
	unsigned char Frac;			// fractal strength
	unsigned char Cliffs;		// cliff / plateau strength
	unsigned char Erosion;
	unsigned char Lakes;			// lake number / size
	unsigned char Climate;
	unsigned char SpecType;		// Mountain, Plateau, Volcano
	unsigned char SpecParam;
} DesignMapPixel;

typedef struct {
	unsigned char ClimateID;		// index of climate in group list
	unsigned char LandTypeID;	// index of land type in climate
} AttributesPixel;

typedef struct {
	unsigned short WaterID;	// identifies the body of water holding this pixel
	unsigned char Type;
	float WaterLevel;
} WaterMapPixel;

typedef struct {
	unsigned char Type;
	unsigned char DoneFlag;
	float WaterLevel;
} AuxWaterMapPixel;

typedef vector3c TerrainNormalPixel;

typedef struct {
	unsigned char r;
	unsigned char g;
	unsigned char b;
} ColourPixel;

#endif