#include "zvarcast.h"
#include <stdafx.h>

#include <Zeolite.h>

#include <zFunc.h>
#include <zList.h>
#include <zVar.h>
#include <zStr.h>

bool zvar_CastBool(ZVAR hVarDest, ZVAR hVarToCast);
bool zvar_CastChar(ZVAR hVarDest, ZVAR hVarToCast); // only permits integer inputs
bool zvar_CastInt(ZVAR hVarDest, long VarID, ZVAR hVarToCast); // does its calc at __int64 precis
bool zvar_CastFloat(ZVAR hVarDest, long VarID, ZVAR hVarToCast); // does its calc at double precis, but supports float too
bool zvar_CastString(ZVAR hVarDest, ZVAR hVarToCast);

bool zvar_Cast(ZVAR hVarDest, long DestVarID, ZVAR hVarSrc) {

	ZVAR hTempVar = NULL;

	switch(DestVarID) { //output type
	case VarID_INVALID:
		return false; // invalid!
	case VarID_void:
		return false; // can't cast to void
	case VarID_variant:
		return false; // not supported yet
	case VarID_bool:
		return zvar_CastBool(hVarDest, hVarSrc);
	case VarID_char:
		return zvar_CastChar(hVarDest, hVarSrc);
	case VarID_byte:
	case VarID_sbyte:
	case VarID_short:
	case VarID_ushort:
	case VarID_int:
	case VarID_uint:
	case VarID_int64:
		return zvar_CastInt(hVarDest, DestVarID, hVarSrc);
	case VarID_float:
	case VarID_double:
		return zvar_CastFloat(hVarDest, DestVarID, hVarSrc);
	case VarID_string:
		return zvar_CastString(hVarDest, hVarSrc);
	case VarID_colour:
	case VarID_vector3c:
	case VarID_vector3i:
	case VarID_vector3f:
	case VarID_vector3d:
	case VarID_buffer:
	case VarID_hvar:
	case VarID_voidptr:
	case VarID_varid:
	case VarID_mapid:
	case VarID_varlist:
	case VarID_map:
	case VarID_format:
	case VarID_filesel:
	case VarID_dirsel:
	case VarID_combosel:
	case VarID_climate:
	case VarID_landtype:
	case VarID_zeofunc:
	case VarID_progbox:
	default:
		return false; // not supported yet
	}

	return false;
}

bool zvar_CastBool(ZVAR hVarDest, ZVAR hVarToCast) {

	bool rval = false;

	BYTE b;
	short s;
	int i;
	__int64 i64;
	float f;
	double d;
	LPVOID lp;
	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = zvar_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		ztype_GetName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastBool error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		zeoWriteToLog(TempStr);
		return false;
	case VarID_variant:
		hTempVar = zvar_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			zeoReportError("zvar_CastBool error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!zvar_CastBool(hVarDest, hTempVar)) {
			zeoWriteToLog("zvar_CastBool error:\r\n - call to VarCastBool failed for variant object");
			return false;
		}
		return true;
	case VarID_bool:
		if(!zvar_GetValueEx(hVarToCast, VarID_bool, &rval)) {
			return false;
		}
		break;
	case VarID_char:
	case VarID_byte:
	case VarID_sbyte:
		if(!zvar_GetValue(hVarToCast, &b)) {
			return false;
		}
		if(b)	rval = true;
		else	rval = false;
		break;
	case VarID_short:
	case VarID_ushort:
		if(!zvar_GetValue(hVarToCast, &s)) {
			return false;
		}
		if(s)	rval = true;
		else	rval = false;
		break;
	case VarID_int:
	case VarID_uint:
	case VarID_varid:
		if(!zvar_GetValue(hVarToCast, &i)) {
			return false;
		}
		if(i)	rval = true;
		else	rval = false;
		break;
	case VarID_int64:
		if(!zvar_GetValue(hVarToCast, &i64)) {
			return false;
		}
		if(i64)	rval = true;
		else	rval = false;
		break;
	case VarID_float:
		if(!zvar_GetValue(hVarToCast, &f)) {
			return false;
		}
		if(f==0.0f)
				rval = false;
		else	rval = true;
		break;
	case VarID_double:
		if(!zvar_GetValue(hVarToCast, &d)) {
			return false;
		}
		if(d==0.0f)
				rval = false;
		else	rval = true;
		break;
	case VarID_voidptr:
		if(!zvar_GetValue(hVarToCast, &lp)) {
			return false;
		}
		if(lp)	rval = true;
		else	rval = false;
		break;
	case VarID_hvar:
		if(!zvar_GetValue(hVarToCast, &lp)) {
			return false;
		}
		if(lp)	rval = true;
		else	rval = false;
		break;
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = zvar_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_bool:
		if(!zvar_SetValue(hVarDest, &rval)) {
			zeoReportError("zvar_CastBool error:\r\n - cannot set value (bool case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = zvar_GetDataPtr(hVarDest))) {
			zeoReportError("zvar_CastBool error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!zvar_IsType(hVar2, VarID_bool)) {
			if(!zvar_ReInit(hVar2, VarID_bool)) {
				zeoReportError("zvar_CastBool error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		if(!zvar_SetValue(hVar2, &rval)) {
			zeoReportError("zvar_CastBool error:\r\n - cannot set value (variant case)");
			return false;
		}
		break;
	default:
		// un-supported variable type
		ztype_GetName(DestVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastBool error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		zeoWriteToLog(TempStr);
		return false;
	}


	return true;
}

bool zvar_CastChar(ZVAR hVarDest, ZVAR hVarToCast) {
	
	char rval = 0;

	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = zvar_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		ztype_GetName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastChar error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		zeoWriteToLog(TempStr);
		return false;
	case VarID_variant:
		hTempVar = zvar_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			zeoReportError("zvar_CastChar error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!zvar_CastChar(hVarDest, hTempVar)) {
			zeoWriteToLog("zvar_CastChar error:\r\n - call to VarCastChar failed for variant object");
			return false;
		}
		return true;
	case VarID_byte:
		{
			BYTE v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	case VarID_sbyte:
		{
			char v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	case VarID_short:
		{
			short v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	case VarID_ushort:
		{
			unsigned short v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	case VarID_int:
		{
			int v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	case VarID_uint:
		{
			unsigned int v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	case VarID_int64:
		{
			__int64 v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (char)v;
		}
		break;
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = zvar_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_char:
		if(!zvar_SetValue(hVarDest, &rval)) {
			zeoReportError("zvar_CastChar error:\r\n - cannot set value (char case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = zvar_GetDataPtr(hVarDest))) {
			zeoReportError("zvar_CastChar error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!zvar_IsType(hVar2, VarID_char)) {
			if(!zvar_ReInit(hVar2, VarID_char)) {
				zeoReportError("zvar_CastChar error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		if(!zvar_SetValue(hVar2, &rval)) {
			zeoReportError("zvar_CastChar error:\r\n - cannot set value (variant/char case)");
			return false;
		}
		break;
	default:
		// un-supported variable type
		ztype_GetName(DestVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastChar error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		zeoWriteToLog(TempStr);
		return false;
	}


	return true;
}

bool zvar_CastInt(ZVAR hVarDest, long VarID, ZVAR hVarToCast) {
	
	__int64 rval = 0;

	switch(VarID) {
	case VarID_byte:
	case VarID_sbyte:
	case VarID_short:
	case VarID_ushort:
	case VarID_int:
	case VarID_uint:
	case VarID_int64:
		break;
	default:
		zeoWriteToLog("zvar_CastInt error:\r\n - unsupported output VarID");
		return false;
	}

	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = zvar_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		ztype_GetName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastInt error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		zeoWriteToLog(TempStr);
		return false;
	case VarID_variant:
		hTempVar = zvar_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			zeoReportError("zvar_CastInt error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!zvar_CastInt(hVarDest, VarID, hTempVar)) {
			zeoWriteToLog("zvar_CastInt error:\r\n - call to VarCastInt failed for variant object");
			return false;
		}
		return true;
	case VarID_byte:
		{
			BYTE v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_char:
	case VarID_sbyte:
		{
			char v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_short:
		{
			short v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_ushort:
		{
			unsigned short v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_int:
	case VarID_mapid:
	case VarID_varid:
		{
			int v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_uint:
		{
			unsigned int v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_int64:
		if(!zvar_GetValue(hVarToCast, &rval)) {
			return false;
		}
		break;
	case VarID_float:
		{
			float v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_double:
		{
			double v;
			if(!zvar_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_string:
		rval = _atoi64(zstr_GetText(hVarToCast));
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = zvar_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_byte:
		{
			unsigned  v = (unsigned char)rval;
			if(!zvar_SetValue(hVarDest, &v)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (byte case)");
				return false;
			}
		}
		break;
	case VarID_sbyte:
		{
			char v = (char)rval;
			if(!zvar_SetValue(hVarDest, &v)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (sbyte case)");
				return false;
			}
		}
		break;
	case VarID_short:
		{
			short v = (short)rval;
			if(!zvar_SetValue(hVarDest, &v)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (short case)");
				return false;
			}
		}
		break;
	case VarID_ushort:
		{
			unsigned short v = (unsigned short)rval;
			if(!zvar_SetValue(hVarDest, &v)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (ushort case)");
				return false;
			}
		}
		break;
	case VarID_int:
		{
			int v = (int)rval;
			if(!zvar_SetValue(hVarDest, &v)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (int case)");
				return false;
			}
		}
		break;
	case VarID_uint:
		{
			unsigned int v = (unsigned int)rval;
			if(!zvar_SetValue(hVarDest, &v)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (uint case)");
				return false;
			}
		}
		break;
	case VarID_int64:
		if(!zvar_SetValue(hVarDest, &rval)) {
			zeoReportError("zvar_CastInt error:\r\n - cannot set value (__int64 case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = zvar_GetDataPtr(hVarDest))) {
			zeoReportError("zvar_CastInt error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!zvar_IsType(hVar2, VarID)) {
			if(!zvar_ReInit(hVar2, VarID)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		switch(VarID) {
		case VarID_byte:
			{
				unsigned char v = (unsigned char)rval;
				if(!zvar_SetValue(hVar2, &v)) {
					zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/byte case)");
					return false;
				}
			}
			break;
		case VarID_sbyte:
			{
				char v = (char)rval;
				if(!zvar_SetValue(hVar2, &v)) {
					zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/sbyte case)");
					return false;
				}
			}
			break;
		case VarID_short:
			{
				short v = (short)rval;
				if(!zvar_SetValue(hVar2, &v)) {
					zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/short case)");
					return false;
				}
			}
			break;
		case VarID_ushort:
			{
				unsigned short v = (unsigned short)rval;
				if(!zvar_SetValue(hVar2, &v)) {
					zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/ushort case)");
					return false;
				}
			}
			break;
		case VarID_int:
			{
				int v = (int)rval;
				if(!zvar_SetValue(hVar2, &v)) {
					zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/int case)");
					return false;
				}
			}
			break;
		case VarID_uint:
			{
				unsigned int v = (unsigned int)rval;
				if(!zvar_SetValue(hVar2, &v)) {
					zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/uint case)");
					return false;
				}
			}
			break;
		case VarID_int64:
			if(!zvar_SetValue(hVar2, &rval)) {
				zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/__int64 case)");
				return false;
			}
			break;
		default:
			zeoReportError("zvar_CastInt error:\r\n - cannot set value (variant/unknown case)");
			return false;
		}
		
		break;
	default:
		// un-supported variable type
		ztype_GetName(DestVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastInt error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		zeoWriteToLog(TempStr);
		return false;
	}


	return true;
}
bool zvar_CastFloat(ZVAR hVarDest, long VarID, ZVAR hVarToCast) {
	
	double rval = 0.0;

	switch(VarID) {
	case VarID_float:
	case VarID_double:
		break;
	default:
		zeoWriteToLog("zvar_CastFloat error:\r\n - unsupported output VarID");
		return false;
	}

	BYTE b;
	char c;
	short s;
	unsigned short us;
	int i;
	unsigned int ui;
	__int64 i64;
	float f;
	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = zvar_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		ztype_GetName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastFloat error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		zeoWriteToLog(TempStr);
		return false;
	case VarID_variant:
		hTempVar = zvar_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			zeoReportError("zvar_CastFloat error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!zvar_CastFloat(hVarDest, VarID, hTempVar)) {
			zeoWriteToLog("zvar_CastFloat error:\r\n - call to VarCastFloat failed for variant object");
			return false;
		}
		return true;
	case VarID_byte:
		if(!zvar_GetValue(hVarToCast, &b)) {
			return false;
		}
		rval = (double)b;
		break;
	case VarID_sbyte:
		if(!zvar_GetValue(hVarToCast, &c)) {
			return false;
		}
		rval = (double)c;
		break;
	case VarID_short:
		if(!zvar_GetValue(hVarToCast, &s)) {
			return false;
		}
		rval = (double)s;
		break;
	case VarID_ushort:
		if(!zvar_GetValue(hVarToCast, &us)) {
			return false;
		}
		rval = (double)us;
		break;
	case VarID_int:
		if(!zvar_GetValue(hVarToCast, &i)) {
			return false;
		}
		rval = (double)i;
		break;
	case VarID_uint:
		if(!zvar_GetValue(hVarToCast, &ui)) {
			return false;
		}
		rval = (double)ui;
		break;
	case VarID_int64:
		if(!zvar_GetValue(hVarToCast, &i64)) {
			return false;
		}
		rval = (double)i64;
		break;
	case VarID_float:
		if(!zvar_GetValue(hVarToCast, &f)) {
			return false;
		}
		rval = (double)f;
		break;
	case VarID_double:
		if(!zvar_GetValue(hVarToCast, &rval)) {
			return false;
		}
		break;
	case VarID_string:
		rval = atof(zstr_GetText(hVarToCast));
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = zvar_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_float:
		f = (float)rval;
		if(!zvar_SetValue(hVarDest, &f)) {
			zeoReportError("zvar_CastFloat error:\r\n - cannot set value (float case)");
			return false;
		}
		break;
	case VarID_double:
		if(!zvar_SetValue(hVarDest, &rval)) {
			zeoReportError("zvar_CastFloat error:\r\n - cannot set value (double case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = zvar_GetDataPtr(hVarDest))) {
			zeoReportError("zvar_CastFloat error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!zvar_IsType(hVar2, VarID)) {
			if(!zvar_ReInit(hVar2, VarID)) {
				zeoReportError("zvar_CastFloat error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		switch(VarID) {
		case VarID_float:
			f = (float)rval;
			if(!zvar_SetValue(hVar2, &f)) {
				zeoReportError("zvar_CastFloat error:\r\n - cannot set value (variant/float case)");
				return false;
			}
			break;
		case VarID_double:
			if(!zvar_SetValue(hVar2, &rval)) {
				zeoReportError("zvar_CastFloat error:\r\n - cannot set value (variant/double case)");
				return false;
			}
			break;
		default:
			zeoReportError("zvar_CastFloat error:\r\n - cannot set value (variant/unknown case)");
			return false;
		}
		
		break;
	default:
		// un-supported variable type
		ztype_GetName(DestVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastFloat error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		zeoWriteToLog(TempStr);
		return false;
	}


	return true;
}
bool zvar_CastString(ZVAR hVarDest, ZVAR hVarToCast) {
	
	CStringA rval = "";

	CzStr zStr;
	CStringA TempStr;
	ZVAR hTempVar;
	
	long SrcVarID = zvar_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		ztype_GetName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastString error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		zeoWriteToLog(TempStr);
		return false;
	case VarID_variant:
		hTempVar = zvar_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			zeoReportError("zvar_CastString error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!zvar_CastString(hVarDest, hTempVar)) {
			zeoWriteToLog("zvar_CastString error:\r\n - call to VarCastString failed for variant object");
			return false;
		}
		return true;
	case VarID_bool:
	case VarID_char:
	case VarID_byte:
	case VarID_sbyte:
	case VarID_short:
	case VarID_ushort:
	case VarID_int:
	case VarID_uint:
	case VarID_int64:
	case VarID_float:
	case VarID_double:
	case VarID_voidptr:
	case VarID_varid:
	case VarID_mapid:
	case VarID_string:
	case VarID_buffer:
	case VarID_colour:
	case VarID_vector3c:
	case VarID_vector3i:
	case VarID_vector3f:
	case VarID_vector3d:
		if(!zvar_GetValueText(hVarToCast, (ZVAR)zStr)) {
			zeoReportError("zvar_CastString error:\r\n - call to var_GetValueA failed");
			return false;
		}
		rval.SetString((char*)zStr);
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = zvar_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_string:
		if(!zvar_SetValue(hVarDest, &rval)) {
			zeoReportError("zvar_CastString error:\r\n - cannot set value (string case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = zvar_GetDataPtr(hVarDest))) {
			zeoReportError("zvar_CastString error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!zvar_IsType(hVar2, VarID_string)) {
			if(!zvar_ReInit(hVar2, VarID_string)) {
				zeoReportError("zvar_CastString error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		if(!zvar_SetValue(hVar2, &rval)) {
			zeoReportError("zvar_CastString error:\r\n - cannot set value (variant case)");
			return false;
		}
		break;
	default:
		// un-supported variable type
		ztype_GetName(DestVarID, (ZSTR)zStr);
		TempStr.Format("zvar_CastString error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		zeoWriteToLog(TempStr);
		return false;
	}

	return true;
}