#include "Zeolite.h"
#include "zmaptile.h"

CzMapTile::CzMapTile(void) {

	m_pTile = NULL;
	m_hMap = NULL;

	// direct interface variables
	m_DirectInterfaceFlag = false;
	m_TileSize = 0;
	m_PixelMemSize = 0;
	m_pMem = NULL;
}

CzMapTile::~CzMapTile(void) {
	Release();
}
bool CzMapTile::Attach(void* pTile, ZMAP hMap) {
	if(!zmap_tile_SetLock(pTile, true)) {
		return false;
	}

	m_pTile = pTile;
	m_hMap = hMap;

	m_DirectInterfaceFlag = false;

	return true;
}
void CzMapTile::Release() {
	if(!m_pTile) {
		return;
	}

	m_DirectInterfaceFlag = false;

	if(!zmap_tile_SetLock(m_pTile, false)) {
		return;
	}

	m_pTile = NULL;
	m_hMap = NULL;
}
bool CzMapTile::GetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		int k = x + y * m_TileSize;
		memcpy(pValue, (char*)m_pMem + k * m_PixelMemSize, m_PixelMemSize);
		return true;
	}

	return zmap_tile_GetPixel(m_pTile, x, y, pValue);
}
bool CzMapTile::SetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		int k = x + y * m_TileSize;
		memcpy((char*)m_pMem + k * m_PixelMemSize, pValue, m_PixelMemSize);
		return true;
	}

	return zmap_tile_SetPixel(m_pTile, x, y, pValue);
}

// direct interface
bool CzMapTile::OpenDirectInterface() {

	// clear to start with, in case of error
	m_DirectInterfaceFlag = false;

	m_pMem = zmap_tile_GetDataPtr(m_pTile);
	if(!m_pMem) {
		zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - cannot get tile data handle");
		return false;
	}

	m_TileSize = zmap_GetTileSize(m_hMap);
	if(!m_TileSize) {
		zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - invalid tile size");
		return false;
	}

	m_PixelMemSize = zmap_GetPixelSize(m_hMap);
	if(!m_PixelMemSize) {
		zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - invalid pixel mem size");
		return false;
	}

	m_DirectInterfaceFlag = true;

	return true;
}
void CzMapTile::CloseDirectInterface() {

	m_DirectInterfaceFlag = false;
}

bool CzMapTile::MarkChanged() {
	return zmap_tile_MarkChanged(m_pTile);
}