#include <Zeolite.h>
#include "zProj.h"
#include <zClasses.h>

//
// init state
//

bool zproj_IsInit() {
	CzVar v;
	v.Create(VarID_bool);
	if(!zscript_Execute("zs", "project.IsInit", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_Init() {
	CzVar v;
	v.Create(VarID_bool);
	if(!zscript_Execute("zs", "project.InitProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_Free() {
	CzVar v;
	v.Create(VarID_bool);
	if(!zscript_Execute("zs", "project.FreeProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}

// loading/saving project

bool zproj_LoadProject(const char* lpProjName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.LoadProject \"");
	s.Append(lpProjName);
	s.Append("\"");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_SaveProject() {
	CzVar v;
	v.Create(VarID_bool);
	if(!zscript_Execute("zs", "project.SaveProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_SaveProjectFile() {		// just save the project file 
	CzVar v;
	v.Create(VarID_bool);
	if(!zscript_Execute("zs", "project.SaveProjectFile", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zproj_SaveProjectAs(const char* lpProjName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.SaveProjectAs \"");
	s.Append(lpProjName);
	s.Append("\"");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_DemandSave() {
	CzStr s;
	if(!zscript_Execute("zs", "project.SaveProjectAsUI", (ZVAR)s)) { 
		return false;
	}
	
	if(s.GetLength())
			return true;
	else	return false;
}

bool zproj_IsPathOK() { 
	if(strlen(zproj_GetProjectFilename()))
		return true;

	return false;
}
const char* zproj_GetProjectFilename() {
	
	ZFUNC hFunc = zfunc_GetFunc("project.GetProjectPath");
	if(!hFunc) {
		zeoReportError("zproj_GetProjectFilename error - function not found");
		return 0;
	}

	ZVAR hRval = 0;
	if(!zfunc_Execute(hFunc, 0, &hRval)) {
		zeoReportError("zproj_GetProjectFilename error - zeofunc failed");
		return 0;
	}

    return zstr_GetText(hRval);
}

//
// saving map
//

bool zproj_AutoSaveMap(const char* lpMapName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.AutoSaveMap \"");
	s.Append(lpMapName);
	s.Append("\"");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_AutoLoadMap(const char* lpMapName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.AutoLoadMap \"");
	s.Append(lpMapName);
	s.Append("\"");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

//
// accessing maps
//

ZLIST zproj_GetMapList() {

	CzFunc f;
	if(!f.GetFunc("project.GetMapList")) {
		zeoReportError("zproj_GetMapList error:\r\n - can't get function handle");
		return NULL;
	}
	if(!f.Execute2()) {
		zeoReportError("zproj_GetMapList error:\r\n - extension function failed");
		return NULL;
	}

	ZVAR hRval = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &hRval)) {
		zeoReportError("zproj_GetMapList error:\r\n - can't get return value");
		return NULL;
	}
	return hRval;
}
ZMAP zproj_GetMap(const char* lpMapName) {
	
	ZFUNC hFunc = zfunc_GetFunc("project.GetMap");
	if(!hFunc) {
		zeoReportError("zproj_GetMap error - function not found");
		return 0;
	}

	ZLIST hArgs = zvar_Create(VarID_varlist);
	if(!hArgs) {
		zeoReportError("zproj_GetMap error:\r\n - cannot create arg list");
		return 0;
	}

	ZVAR hMapName = zlist_CreateItem(hArgs, VarID_string, "MapName");
	if(!zstr_SetText(hMapName, lpMapName)) {
		zeoReportError("zproj_GetMap error:\r\n - cannot set MapName value");
		return 0;
	}

	ZVAR hRval1 = 0, hRval2 = 0;
	if(!zfunc_Execute(hFunc, hArgs, &hRval1)) {
		return 0;
	}
	if(!zvar_GetValueEx(hRval1, VarID_hvar, &hRval2)) {
		return 0;
	}
    
	// cleanup
	zvar_Delete(hArgs);

	return hRval2;
}
ZMAP zproj_CreateMap(const char* lpMapName) {
	
	ZFUNC hFunc = zfunc_GetFunc("project.CreateMap");
	if(!hFunc) {
		zeoReportError("zproj_CreateMap error - function not found");
		return 0;
	}

	ZLIST hArgs = zvar_Create(VarID_varlist);
	if(!hArgs) {
		zeoReportError("zproj_CreateMap error:\r\n - cannot create arg list");
		return 0;
	}

	ZVAR hMapName = zlist_CreateItem(hArgs, VarID_string, "MapName");
	if(!zstr_SetText(hMapName, lpMapName)) {
		zeoReportError("zproj_CreateMap error:\r\n - cannot set MapName value");
		return 0;
	}

	ZVAR hRval1 = 0, hRval2 = 0;
	if(!zfunc_Execute(hFunc, hArgs, &hRval1)) {
		return 0;
	}
	if(!zvar_GetValueEx(hRval1, VarID_hvar, &hRval2)) {
		return 0;
	}
    
	// cleanup
	zvar_Delete(hArgs);

	return hRval2;
	
}
ZMAP zproj_GetOrCreateMap(const char* lpMapName) {
	
	ZMAP hMap = NULL;

	if(hMap = zproj_GetMap(lpMapName)) {
		return hMap;
	}

	if(hMap = zproj_CreateMap(lpMapName)) {
		return hMap;
	}

	zeoReportError("zproj_GetOrCreateMap error:\r\n - can neither get nor create map");
	return 0;
	
}
bool zproj_GetMapDisplayName(ZSTR hDest, const char* lpMapName) {

	// to be safe, set return value to be value of argument at start
	if(!zstr_SetText(hDest, lpMapName)) {
		zeoReportError("zproj_GetMapDisplayName error:\r\n - can't set return value");
		return false;
	}
		
	CzFunc f;
	if(!f.GetFunc("project.GetMapDisplayName")) {
		zeoReportError("zproj_GetMapDisplayName error:\r\n - cannot get 'project.GetMapDisplayName' function");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpMapName)) {
		return false;
	}

	if(!f.Execute2()) {
		zeoReportError("zproj_GetMapDisplayName error:\r\n - call to 'project.GetMapDisplayName' failed");
		return false;
	}

	if(!zvar_CopyValue(hDest, f.m_rval)) {
		zeoReportError("zproj_GetMapDisplayName error:\r\n - can't set return value");
		return false;
	}

	const char* lpLongName = zstr_GetText(hDest);
	if(!lpLongName) {
		return false;
	}
	if(!strlen(lpLongName)) {
		return false;
	}
	

	return true;
}

//
// settings and options
//

ZLIST zproj_GetSettingsList() {

	CzFunc f;
	if(!f.GetFunc("project.GetSettingsList")) {
		zeoReportError("zproj_GetSettingsList error:\r\n - can't get function handle");
		return NULL;
	}
	if(!f.Execute2()) {
		zeoReportError("zproj_GetSettingsList error:\r\n - extension function failed");
		return NULL;
	}

	ZVAR hRval;
	if(!f.m_rval.GetValue(VarID_hvar, &hRval)) {
		zeoReportError("zproj_GetSettingsList error:\r\n - can't get return value");
		return NULL;
	}
	return hRval;
	
}
bool zproj_GetSetting(const char* lpName, long VarID, void* pValue) {
	ZLIST hSettings = zproj_GetSettingsList();
	if(!hSettings) {
		return false;
	}

	ZVAR hVar = zlist_GetItemA(hSettings, lpName);
	if(!hVar) {
		return false;
	}

	if(!zvar_GetValueEx(hVar, VarID, pValue)) {
		return false;
	}

	return true;
}

bool zproj_SetSetting(const char* lpName, long VarID, void* pValue) {
	ZLIST hSettings = zproj_GetSettingsList();
	if(!hSettings) {
		return false;
	}

	ZVAR hVar = zlist_GetItemA(hSettings, lpName);
	if(!hVar) {
		return false;
	}

	if(!zvar_SetValueEx(hVar, VarID, pValue)) {
		return false;
	}

	return true;
}

bool zproj_GetOpFlag(const char* lpOpName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.GetOpFlag \"");
	s.Append(lpOpName);
	s.Append("\"");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_SetOpFlag(const char* lpOpName, bool StateFlag) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.SetOpFlag \"");
	s.Append(lpOpName);
	s.Append("\" ");
	if(StateFlag)
			s.Append("true");
	else	s.Append("false");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zproj_DelOpsFollowing(const char* lpOpName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.DelOpsFollowing \"");
	s.Append(lpOpName);
	s.Append("\"");

	if(!zscript_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zproj_GetHeightfieldRange(float& minval, float& maxval) {
	
	ZFUNC hFunc = zfunc_GetFunc("project.GetHeightfieldRange");
	if(!hFunc) {
		zeoReportError("zproj_GetHeightfieldRange error - function not found");
		return false;
	}

	ZLIST hRval = 0;
	if(!zfunc_Execute(hFunc, 0, &hRval)) {
		zeoReportError("zproj_GetHeightfieldRange error - zeofunc failed");
		return false;
	}

	if(!zvar_GetValue(zlist_GetItemA(hRval, "MinAlt"), &minval)) {
		zeoReportError("zproj_GetHeightfieldRange error - cannot get min alt");
		return false;
	}

	if(!zvar_GetValue(zlist_GetItemA(hRval, "MaxAlt"), &maxval)) {
		zeoReportError("zproj_GetHeightfieldRange error - cannot get max alt");
		return false;
	}

	return true;
}

bool zproj_SetMapFlag(const char* lpMapName, long MapFlag, bool State) {
	ZMAP m = zproj_GetMap(lpMapName);
	if(!m) {
		return false;
	}

	return zmap_SetFlag(m, MapFlag, State);
}


long zproj_GetCurTileSizeDM() {
	
	CzFunc f;
	if(!f.GetFunc("project.GetCurTileSizeDM")) {
		return 0;
	}
	if(!f.Execute()) {
		return 0;
	}

	long rval = 0;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		return 0;
	}

	return rval;
}
bool zproj_SetCurTileSizeDM(long TileSizeDM) {

	CzFunc f;
	if(!f.GetFunc("project.SetCurTileSizeDM")) {
		return false;
	}

	if(!f.m_args.SetItemValueI(0, VarID_int, &TileSizeDM)) {
		return false;
	}
	if(!f.Execute()) {
		return false;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		return false;
	}
	return rval;
}

float zproj_GetCurHorizScaleHF() {

	CzFunc f;
	if(!f.GetFunc("project.GetCurHorizScaleHF")) {
		return 0.0f;
	}
	if(!f.Execute()) {
		return 0.0f;
	}

	float rval = 0.0f;
	if(!f.m_rval.GetValue(VarID_float, &rval)) {
		return 0.0f;
	}

	return rval;
}
bool zproj_SetCurHorizScaleHF(float HorizScaleHF) {

	CzFunc f;
	if(!f.GetFunc("project.SetCurHorizScaleHF")) {
		return NULL;
	}

	if(!f.m_args.SetItemValue((long)0, VarID_float, &HorizScaleHF)) {
		return NULL;
	}
	if(!f.Execute()) {
		return NULL;
	}

	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		return NULL;
	}
	return rval;
}

long zproj_nClimates() {
	CzFunc f;
	if(!f.GetFunc("project.nClimates")) {
		return 0;
	}
	if(!f.Execute2()) {
		return 0;
	}

	long rval = 0;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		return 0;
	}
	return rval;
}
ZVAR zproj_GetClimate(long CliID) {
	CzFunc f;
	if(!f.GetFunc("project.GetClimate")) {
		return NULL;
	}
	if(!f.SetArgValueI(0, VarID_int, &CliID)) {
		return NULL;
	}
	if(!f.Execute2()) {
		return NULL;
	}

	ZVAR hCli = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &hCli)) {
		return NULL;
	}
	return hCli;
}
ZVAR zproj_GetClimateA(const char* lpCliName) {
	CzFunc f;
	if(!f.GetFunc("project.GetClimateA")) {
		return NULL;
	}
	if(!zstr_SetText(f.GetArgI(0), lpCliName)) {
		return NULL;
	}
	if(!f.Execute2()) {
		return NULL;
	}

	ZVAR hCli = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &hCli)) {
		return NULL;
	}
	return hCli;
}
long zproj_GetClimateIndex(const char* lpCliName) {
	CzFunc f;
	if(!f.GetFunc("project.GetClimateIndex")) {
		return -1;
	}
	if(!zstr_SetText(f.GetArgI(0), lpCliName)) {
		return -1;
	}
	if(!f.Execute2()) {
		return -1;
	}

	long rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		return -1;
	}
	return rval;
}
long zproj_AddClimateA(const char* lpCliName) {
	CzFunc f;
	if(!f.GetFunc("project.AddClimateToProject")) {
		return -1;
	}
	if(!zstr_SetText(f.GetArgI(0), lpCliName)) {
		return -1;
	}
	if(!f.Execute2()) {
		return -1;
	}

	long rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		return -1;
	}
	return rval;
}