#include "Zeolite.h"
#include "zvar.h"

CzVar::CzVar(void) {
	m_hVar = 0;
	m_AutoDelete = false;
}
CzVar::CzVar(ZVAR hVar) {
	m_hVar = hVar;
	m_AutoDelete = false;
}
CzVar::CzVar(long VarID, const char* lpVarName) {
	m_hVar = 0;
	m_AutoDelete = false;

	if(lpVarName) {
		CreateShared(VarID, lpVarName);
	} else {
		Create(VarID);
	}
}

CzVar::~CzVar(void) {
	if(m_hVar && m_AutoDelete) {
		Delete();
	}
}

bool CzVar::Attach(ZVAR hVar) {

	if(m_hVar && m_AutoDelete) Delete();

	if(!hVar)
		return false;
	
	m_hVar = hVar;
	m_AutoDelete = false;
	return true;
}

ZVAR CzVar::GetZVAR() { return m_hVar; }

bool CzVar::Create(long VarID) {
	if(m_hVar && m_AutoDelete) Delete();

	m_hVar = zvar_Create(VarID);

	if(!m_hVar)	return false;

	m_AutoDelete = true;

	return true;
}
bool CzVar::CreateShared(long VarID, const char* lpVarName) {
	if(m_hVar && m_AutoDelete) Delete();

	m_hVar = zvar_CreateShared(VarID, lpVarName);

	if(!m_hVar)	return false;

	m_AutoDelete = true;

	return true;
}

bool CzVar::GetSharedVar(const char* lpVarName) {
	if(m_hVar && m_AutoDelete) Delete();

	if(!Attach(zvar_GetSharedVar(lpVarName)))
		return false;

	if(!m_hVar)	
		return false;

	return true;
}

bool CzVar::Delete() {
	if(!m_hVar) return true;

	bool rval = false;
	
	rval = zvar_Delete(m_hVar);
	
	m_hVar = 0;
	return rval;
}
void CzVar::Release() {
	m_hVar = 0;
}

const char* CzVar::GetName() {
	if(!m_hVar)	return 0;

	return zvar_GetName(m_hVar);
}
bool CzVar::Rename(const char* lpNewName) {
	if(!m_hVar)	return false;

	return zvar_Rename(m_hVar, lpNewName);
}

long CzVar::GetTypeID() {
	if(!m_hVar)	return false;

	return zvar_GetTypeID(m_hVar);
}
bool CzVar::GetTypeName(ZSTR hTypeName) {
	if(!m_hVar)	return false;

	return ztype_GetName(GetTypeID(), hTypeName);
}
bool CzVar::IsType(long VarID) {
	if(!m_hVar) {
		if(VarID==VarID_INVALID)
				return true;
		else	return false;
	}

	return zvar_IsType(m_hVar, VarID);
}
bool CzVar::ReInit(long VarID) {
	if(!m_hVar)	return false;

	return zvar_ReInit(m_hVar, VarID);
}

bool CzVar::GetValue(long VarID, void* pValue) {
	if(!m_hVar)	return false;

	return zvar_GetValueEx(m_hVar, VarID, pValue);
}
bool CzVar::SetValue(long VarID, void* pValue) {
	if(!m_hVar)	return false;

	return zvar_SetValueEx(m_hVar, VarID, pValue);
}
bool CzVar::GetValueText(ZSTR hStrVar) {
	if(!m_hVar)	return false;

	return zvar_GetValueText(m_hVar, hStrVar);
}
bool CzVar::SetValueText(const char* lpValueText) {
	if(!m_hVar)	return false;

	return zvar_SetValueText(m_hVar, lpValueText);
}

bool CzVar::CopyValue(ZVAR hVarSrc) {
	if(!m_hVar)	return false;

	return zvar_CopyValue(m_hVar, hVarSrc);
}

void* CzVar::GetDataPtr(long VarID) {
	if(!m_hVar)	return NULL;

	if(VarID!=GetTypeID()) {
		return NULL;
	}

	return zvar_GetDataPtr(m_hVar);
}

bool CzVar::SetVarRef(ZVAR hSrcVar) {
	if(!m_hVar)	return false;

	return zvar_SetVarRef(m_hVar, hSrcVar);
}

bool CzVar::EditUI(const char* lpPrompt, unsigned long Flags, ZLIST hOptions) {
	if(!m_hVar)	return false;

	return zvar_EditUI(m_hVar, lpPrompt, Flags, hOptions);
}