#include <Zeolite.h>
#include "zView.h"
#include <zClasses.h>
#include <zFunc.h>

bool zview_ShowMap(const char* lpMapName) {
	
	CzFunc f;
	if(!f.GetFunc("view.ShowMap")) {
		zeoReportError("zview_ShowMap error:\r\n - cannot get function handle");
		return false;
	}
	
	if(!zstr_SetText(f.GetArgI(0), lpMapName)) {
		zeoReportError("zview_ShowMap error:\r\n - cannot set arg");
		return false;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_ShowMap error:\r\n - function call failed");
		return false;
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zview_ShowMap error:\r\n - cannot get rval");
		return false;
	}
	return rval;
}
bool zview_Refresh(bool RedrawBuffer) {
	CzFunc f;
	if(!f.GetFunc("view.Refresh")) {
		zeoReportError("zview_Refresh error:\r\n - cannot get function handle");
		return false;
	}
	
	if(!f.SetArgValueI(0, VarID_bool, &RedrawBuffer)) {
		zeoReportError("zview_Refresh error:\r\n - cannot set arg");
		return false;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_Refresh error:\r\n - function call failed");
		return false;
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zview_Refresh error:\r\n - cannot get rval");
		return false;
	}
	return rval;
}
ZMAP zview_GetActiveMap() {
	
	CzFunc f;
	if(!f.GetFunc("view.GetActiveMap")) {
		zeoReportError("zview_GetActiveMap error:\r\n - cannot get function handle");
		return NULL;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_GetActiveMap error:\r\n - function call failed");
		return NULL;
	}
	
	ZVAR rval = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &rval)) {
		zeoReportError("zview_GetActiveMap error:\r\n - cannot get rval");
		return NULL;
	}
	return rval;
}
bool zview_GetSelectedArea(ZSTR hMapName, LPRECT lpRect) {
	
	CzFunc f;
	if(!f.GetFunc("view.GetSelectedArea")) {
		zeoReportError("zview_GetSelectedArea error:\r\n - cannot get function handle");
		return false;
	}

	if(!f.SetArgValueI(0, VarID_hvar, &hMapName)) {
		return false;
	}

	CzVar RectVar(VarID_rect32);
	ZVAR hRectVar = (ZVAR)RectVar;

	if(!f.SetArgValueI(1, VarID_hvar, &hRectVar)) {
		return false;
	}
	
	if(!f.Execute2()) {
		return false;
	}

	if(lpRect) {
		rect32 r32;
		RectVar.GetValue(VarID_rect32, &r32);

		SetRect(lpRect, r32.x1, r32.y2, r32.x2, r32.y1);
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		return false;
	}
	return rval;
}
bool zview_SetSelectedArea(const char* lpMapName, LPRECT lpRect) {
	
	CzFunc f;
	if(!f.GetFunc("view.SetSelectedArea")) {
		zeoReportError("zview_SetSelectedArea error:\r\n - cannot get function handle");
		return false;
	}

	if(!zstr_SetText(f.GetArgI(0), lpMapName)) {
		return false;
	}
	
	rect32 r32;
	r32.x1 = min(lpRect->left, lpRect->right);
	r32.x2 = max(lpRect->left, lpRect->right);
	r32.y1 = min(lpRect->top, lpRect->bottom);
	r32.y2 = max(lpRect->top, lpRect->bottom);
	f.SetArgValueI(1, VarID_rect32, &r32);
	
	if(!f.Execute2()) {
		return false;
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		return false;
	}
	return rval;
}

bool zview_SetMouseMode(long mode) {

	CzFunc f;
	if(!f.GetFunc("view.SetMouseMode")) {
		zeoReportError("zview_SetMouseMode error:\r\n - cannot get function handle");
		return false;
	}
	
	if(!f.SetArgValueI(0, VarID_int, &mode)) {
		zeoReportError("zview_SetMouseMode error:\r\n - cannot set arg");
		return false;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_SetMouseMode error:\r\n - function call failed");
		return false;
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zview_SetMouseMode error:\r\n - cannot get rval");
		return false;
	}
	return rval;
}

long zview_GetBrushRadius() {

	CzFunc f;
	if(!f.GetFunc("view.GetBrushRadius")) {
		zeoReportError("zview_SetBrushRadius error:\r\n - cannot get function handle");
		return 0;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_GetBrushRadius error:\r\n - function call failed");
		return 0;
	}
	
	long rval = 0;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zview_GetBrushRadius error:\r\n - cannot get rval");
		return 0;
	}
	return rval;
}
bool zview_SetBrushRadius(long radius) {

	CzFunc f;
	if(!f.GetFunc("view.SetBrushRadius")) {
		zeoReportError("zview_SetBrushRadius error:\r\n - cannot get function handle");
		return false;
	}
	
	if(!f.SetArgValueI(0, VarID_int, &radius)) {
		zeoReportError("zview_SetBrushRadius error:\r\n - cannot set arg");
		return false;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_SetBrushRadius error:\r\n - function call failed");
		return false;
	}
	
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zview_SetBrushRadius error:\r\n - cannot get rval");
		return false;
	}
	return rval;
}
void zview_EnableDraw(bool EnableFlag) {

	CzFunc f;
	if(!f.GetFunc("view.EnableDraw")) {
		zeoReportError("zview_EnableDraw error:\r\n - cannot get function handle");
		return;
	}
	
	if(!f.SetArgValueI(0, VarID_bool, &EnableFlag)) {
		zeoReportError("zview_EnableDraw error:\r\n - cannot set arg");
		return;
	}
	
	if(!f.Execute2()) {
		zeoReportError("zview_EnableDraw error:\r\n - function call failed");
		return;
	}
}